# -*- coding: utf-8 -*-

# Path hack as shown by Kenneth Reitz at http://kennethreitz.com/repository-structure-and-python.html

import os
import sys
sys.path.insert(0, os.path.abspath('..'))

import nikola  # NOQA
