# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Również dostępny w",
    "Archive": "Archiwum",
    "Categories": "Kategorie",
    "LANGUAGE": "polski",
    "More posts about": "Więcej postów o",
    "Newer posts": "Nowsze posty",
    "Next post": "Następny post",
    "Older posts": "Starsze posty",
    "Original site": "Oryginalna strona",
    "Posted": "Opublikowany",
    "Posts about %s": "Posty o %s",
    "Posts for year %s": "Posty z roku %s",
    "Posts for {month} {year}": "Posty z {month} {year}",
    "Previous post": "Poprzedni post",
    "Read in English": "Czytaj po polsku",
    "Read more": "Czytaj więcej",
    "Source": "Źródło",
    "Tags and Categories": "Tagi i Kategorie",
    "Tags": "Tags",
    "old posts page %d": "stare posty, strona %d",
}
