# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "LANGUAGE": "Ελληνικά",
    "Posts for year %s": "Αναρτήσεις για τη χρονιά %s",
    "Archive": "Αρχείο",
    "Posts about %s": "Αναρτήσεις για %s",
    "Tags": "Ετικέτες",
    "Also available in": "Διαθέσιμο και στο",
    "More posts about": "Περισσότερες αναρτήσεις για",
    "Posted": "Αναρτήθηκε",
    "Original site": "Ιστοσελίδα αρχικής ανάρτησης",
    "Read in English": "Διαβάστε στα Ελληνικά",
    "Newer posts": "Νεότερες αναρτήσεις",
    "Older posts": "Παλαιότερες αναρτήσεις",
    "Previous post": "Προηγούμενη ανάρτηση",
    "Next post": "Επόμενη ανάρτηση",
    "old posts page %d": "σελίδα παλαιότερων αναρτήσεων %d",
    "Read more": "Read more",
    "Source": "Source",
}
