# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "همچنین قابل دسترس از",
    "Archive": "آرشیو",
    "LANGUAGE": "فارسی",
    "More posts about": "ارسال‌های بیشتر دربارهٔ",
    "Newer posts": "ارسال‌های جدید‌تر",
    "Next post": "ارسال بعدی",
    "Older posts": "پست‌های قدیمی‌تر",
    "Original site": "سایت اصلی",
    "Posted": "ارسال شده",
    "Posts about %s": "ارسال‌ها دربارهٔ %s",
    "Posts for year %s": "ارسال‌ها برای سال %s",
    "Posts for {month} {year}": "ارسال برای {month} {year}",
    "Previous post": "ارسال پیشین",
    "Read in English": "به فارسی بخوانید",
    "Read more": "بیشتر بخوانید",
    "Source": "منبع",
    "Tags": "برچسب‌ها",
    "old posts page %d": "صفحهٔ ارسال‌های قدیمی %d",
}
