# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Също достъпно в",
    "Archive": "Архив",
    "Categories": "Категории",
    "LANGUAGE": "Български",
    "More posts about": "Още публикации относно ",
    "Newer posts": "Нови публикации",
    "Next post": "Следваща публикация",
    "Older posts": "Стари публикации",
    "Original site": "Оригиналния сайт",
    "Posted": "Публиковано",
    "Posts about %s": "Публикации относно %s",
    "Posts for year %s": "Публикации за %s година",
    "Posts for {month} {year}": "Публикации за {month} {year}",
    "Previous post": "Предишна публикация",
    "Read in English": "Прочетете на български",
    "Read more": "Прочети още",
    "Source": "Source",
    "Tags and Categories": "Тагове и Категории",
    "Tags": "Тагове",
    "old posts page %d": "стари публикации страница %d",
}
