# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Şu dilde de mevcut",
    "Archive": "Arşiv",
    "LANGUAGE": "Türkçe",
    "More posts about": "İlgili diğer yazılar",
    "Newer posts": "Daha yeni yazılar",
    "Next post": "Sonraki yazı",
    "Older posts": "Daha yeni yazılar",
    "Original site": "Orjinal site",
    "Posted": "Yayın tarihi",
    "Posts about %s": "Yazılar hakkında %s",
    "Posts for year %s": "Yıla göre yazılar %s",
    "Posts for {month} {year}": "{month} {year} göre yazılar",
    "Previous post": "Önceki yazı",
    "Read in English": "İngilizce olarak oku",
    "Read more": "Devamını oku",
    "Source": "Kaynak",
    "Tags": "Etiketler",
    "old posts page %d": "Geçmiş yazılar sayfası %d",
}
