# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Также доступно в",
    "Archive": "Архив",
    "LANGUAGE": "Русский",
    "More posts about": "Больше записей о",
    "Newer posts": "Новые записи",
    "Next post": "Следующая запись",
    "Older posts": "Старые записи",
    "Original site": "Оригинальный сайт",
    "Posted": "Опубликовано",
    "Posts about %s": "Записи с тэгом %s:",
    "Posts for year %s": "Записи за %s год",
    "Posts for {month} {year}": "",
    "Previous post": "Предыдущая запись",
    "Read in English": "Прочесть по-русски",
    "Read more": "Продолжить чтение",
    "Source": "Source",
    "Tags": "Тэги",
    "old posts page %d": "страница со старыми записями %d",
}
