# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "",
    "Archive": "",
    "Categories": "",
    "LANGUAGE": "",
    "More posts about": "",
    "Newer posts": "",
    "Next post": "",
    "Older posts": "",
    "Original site": "",
    "Posted": "",
    "Posts about %s": "",
    "Posts for year %s": "",
    "Posts for {month} {year}": "",
    "Previous post": "",
    "Read in English": "",
    "Read more": "",
    "Source": "",
    "Tags and Categories": "",
    "Tags": "",
    "old posts page %d": "",
}
