# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Disponible aussi en",
    "Archive": "Archives",
    "LANGUAGE": "Français",
    "More posts about": "Plus de billets sur",
    "Newer posts": "Billets récents",
    "Next post": "Billet suivant",
    "Older posts": "Anciens billets",
    "Original site": "Site d'origine",
    "Posted": "Publié",
    "Posts about %s": "Billets sur %s",
    "Posts for year %s": "Billets de l'année %s",
    "Posts for {month} {year}": "Billets de {month} {year}",
    "Previous post": "Billet précédent",
    "Read in English": "Lire en français",
    "Read more": "Lire la suite",
    "Source": "Source",
    "Tags": "Étiquettes",
    "old posts page %d": "anciens billets page %d",
}
